package net.wanji.web.task;

import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.dto.SystemServiceStatusDTO;
import net.wanji.common.utils.tool.SocketUtil;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.web.common.constant.Constant;
import net.wanji.web.common.util.date.DateStyle;
import net.wanji.web.websocket.WebSocketServer;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/03/21 10:42
 */
@Component
@Slf4j
public class SystemServiceStatusTask {

    private static final String LOCAL_IP = "172.17.0.1";
    private static final String HIK_LOCAL_IP = "10.102.1.183";


    public static final List<SystemServiceStatusDTO> sysList = new ArrayList<>();

    static {
        sysList.add(new SystemServiceStatusDTO("MYSQL", LOCAL_IP, 3306, 30000, 0, null, null));
        sysList.add(new SystemServiceStatusDTO("UTC", LOCAL_IP, 32000, 30000, 0, null, null));
        sysList.add(new SystemServiceStatusDTO("海康", HIK_LOCAL_IP, 80, 30000, 0, null, null));
    }

    @Scheduled(fixedRate = 60 * 1000)
    public void telnetStatus() throws Exception {
        Set<WebSocketServer> webSocketSet = WebSocketServer.getWebSocketSet(Constant.WEBSOCKET_TOPIC_SYS_SERVICE_STATUS);
        if (!CollectionUtils.isEmpty(webSocketSet)) {
            for (SystemServiceStatusDTO dto : sysList) {
                String ip = dto.getIp();
                int port = dto.getPort();
                int timeout = dto.getTimeout();
                int lastStatus = dto.getStatus();
                int currentStatus = SocketUtil.telnet(ip, port, timeout) ? 0 : 1;
                if (currentStatus != lastStatus) {
                    dto.setStatus(currentStatus);
                }
                // 如果正常 每分钟更新状态； 异常 更新异常初始状态
                if (StringUtils.isBlank(dto.getStartTime()) || dto.getStatus() == 0) {
                    LocalDateTime now = LocalDateTime.now();
                    String format = now.format(DateTimeFormatter.ofPattern(DateStyle.YYYY_MM_DD_HH_MM_SS.getValue()));
                    dto.setStartTime(format);
                    dto.setStartDate(new Date());
                }
            }
            List<SystemServiceStatusDTO> sort = sysList.stream().
                    sorted(Comparator.comparing(SystemServiceStatusDTO::getStartDate)).collect(Collectors.toList());
            WebSocketServer.sendTopicInfo2All(JSON.toJSONString(sort), Constant.WEBSOCKET_TOPIC_SYS_SERVICE_STATUS);
        }
    }

}
